from .control_client import ControlClient
from .core import ApiClient
from .jps_client import JpsClient


class JelasticClientFactory:
    def __init__(self, api_url: str, api_token: str):
        self.api_client = ApiClient(api_url, api_token)

    def create_jps_client(self) -> "JpsClient":
        return JpsClient(self.api_client)

    def create_control_client(self) -> "ControlClient":
        return ControlClient(self.api_client)



import pytest

from jelastic_client import ControlClient
from jelastic_client.core import JelasticClientException


def test_control_client_delete_non_existent_environment_raises_exception(
        control_client: ControlClient,
        non_existent_env_name: str):
    # Arrange

    # Act / Assert
    with pytest.raises(JelasticClientException):
        control_client.delete_env(non_existent_env_name)



import os

import pytest

from jelastic_client import ControlClient, JpsClient
from jelastic_client.core.exceptions import JelasticClientException


def test_jps_client_install_valid_manifest_creates_environment(
        control_client: ControlClient,
        jps_client: JpsClient,
        test_data_dir: str,
        new_env_name: str):
    # Arrange
    filename = os.path.join(test_data_dir, "valid_manifest.jps")

    # Act
    jps_client.install(filename, new_env_name)

    # Assert
    assert control_client.env_exists(new_env_name)


def test_jps_client_install_valid_manifest_makes_environment_run(
        control_client: ControlClient,
        jps_client: JpsClient,
        test_data_dir: str,
        new_env_name: str):
    # Arrange
    filename = os.path.join(test_data_dir, "valid_manifest.jps")

    # Act
    jps_client.install(filename, new_env_name)

    # Assert
    assert control_client.env_is_running(new_env_name)


def test_jps_client_install_invalid_manifest_raises_exception(
        jps_client: JpsClient,
        test_data_dir: str,
        new_env_name: str):
    # Arrange
    filename = os.path.join(test_data_dir, "invalid_manifest.jps")

    # Act / Assert
    with pytest.raises(JelasticClientException):
        jps_client.install(filename, new_env_name)


def test_jps_client_get_engine_version_returns_supported_engine_version(
        jps_client: JpsClient,
        supported_jelastic_version: str):
    # Arrange

    # Act
    actual_version = jps_client.get_engine_version()

    # Assert
    assert supported_jelastic_version == actual_version



from typing import Dict


class JelasticClientException(Exception):
    """
    Generic Jelastic Client Exception
    """

    def __init__(self, message: str, response: Dict):
        super().__init__(message)
        self.response = response


class ApiClientException(JelasticClientException):
    """
    Low-level API Exception
    """

    def __init__(self, message: str, response: Dict):
        super().__init__(message, response)



from .core import ApiClient, BaseClient, JelasticClientException, success_response


class JpsClient(BaseClient):
    grp_cls = "marketplace.jps"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def install(self, filename: str, env_name: str) -> None:
        with open(filename) as file:
            manifest_content = file.read()

            response = self.execute(
                "Install",
                jps=manifest_content,
                envName=env_name
            )

            if not success_response(response):
                raise JelasticClientException(
                    f"installation of manifest {filename} failed", response)

    def get_engine_version(self) -> str:
        response = self.execute(
            "GetEngineVersion"
        )

        if not success_response(response):
            raise JelasticClientException(
                f"getting engine version failed", response)

        return response["version"]



from enum import Enum
from typing import Dict

from .core import ApiClient, BaseClient, JelasticClientException, success_response, ApiClientException


class Status(Enum):
    Running = 1
    Down = 2
    Launching = 3
    Sleep = 4
    Unknown = 5
    Creating = 6
    Cloning = 7
    NotExists = 8
    Exporting = 9
    Migrating = 10
    Broken = 11
    Updating = 12
    Stopping = 13
    GoingToSleep = 14
    Restoring = 15


class ControlClient(BaseClient):
    grp_cls = "environment.control"

    def __init__(self, api_client: ApiClient):
        super().__init__(api_client)

    def delete_env(self, env_name: str) -> None:
        response = self.execute(
            "DeleteEnv",
            envName=env_name
        )

        if not success_response(response):
            raise JelasticClientException(
                f"deletion of environment {env_name} failed", response)

    def get_env_info(self, env_name: str) -> Dict:
        try:
            response = self.execute(
                "GetEnvInfo",
                envName=env_name
            )
            return response
        except ApiClientException as e:
            return {
                "result": e.response["result"],
                "env": {
                    "status": Status.NotExists
                }
            }

    def env_is_running(self, env_name: str) -> bool:
        env_info = self.get_env_info(env_name)
        env_status = Status(env_info["env"]["status"])

        return env_status == Status.Running

    def env_exists(self, env_name: str) -> bool:
        env_info = self.get_env_info(env_name)
        env_status = Status(env_info["env"]["status"])

        return env_status is not Status.NotExists and env_status is not Status.Unknown



from abc import ABC, abstractmethod
from typing import Dict

from . import ApiClient


def success_response(response: Dict) -> bool:
    return response["result"] == 0


class BaseClient(ABC):

    def __init__(self, api_client: ApiClient):
        self.api_client = api_client

    @property
    @abstractmethod
    def grp_cls(self):
        raise NotImplementedError

    def execute(self, fnc: str, **kwargs) -> Dict:
        response = self.api_client.execute(
            self._fnc(fnc),
            **kwargs
        )
        return response

    def _fnc(self, fnc_name: str):
        return f"{self.grp_cls}.{fnc_name}"
